#include "StdAfx.h"
#include "EmbededNSB.h"

CEmbededNSB::CEmbededNSB()
{
	m_pNSBHorz = NULL;
	m_pNSBVert = NULL;
	m_pWndProcOld = NULL;
	m_lrHitTest = HTNOWHERE;
	m_rcNSBHorz.SetRectEmpty();
	m_rcNSBVert.SetRectEmpty();
	m_pRootContainerHorz = NULL;
	m_pRootContainerVert = NULL;
}

CEmbededNSB::~CEmbededNSB()
{
}

LRESULT CALLBACK CEmbededNSB::_ENSBProc( HWND hWnd, UINT iMsg, WPARAM wParam, LPARAM lParam )
{
	ATLASSERT(::IsWindow(hWnd));
	if(!::IsWindow(hWnd)) return FALSE;

	CEmbededNSB* pENSB = (CEmbededNSB*)::GetProp(hWnd, ENSB_PROP);
	ATLASSERT(pENSB);
	if(!pENSB) return S_FALSE;

	switch(iMsg)
	{
	case WM_NCHITTEST:
		{
			return pENSB->OnNCHitTest(wParam, lParam);
		}
		break;
	case WM_NCLBUTTONDOWN:
		{
			return pENSB->OnNCLBtnDn(wParam, lParam);
		}
		break;
	case WM_LBUTTONDOWN:
		{
			return pENSB->OnLBtnDn(wParam, lParam);
		}
		break;
	case WM_LBUTTONUP:
		{
			return pENSB->OnLBtnUp(wParam, lParam);
		}
		break;
	case WM_MOUSEMOVE:
		{
			return pENSB->OnMouseMove(wParam, lParam);
		}
		break;
	case WM_NCMOUSEMOVE:
		{
			return pENSB->OnNCMouseMove(wParam, lParam);
		}
		break;
	case WM_NCLBUTTONDBLCLK:
		{
			return pENSB->OnNCLBtnDblClk(wParam, lParam);
		}
		break;
	case WM_NCRBUTTONDOWN:
		{
			return pENSB->OnNCRBtnDn(wParam, lParam);
		}
		break;
	case WM_NCRBUTTONDBLCLK:
		{
			return pENSB->OnNCRBtnDblClk(wParam, lParam);
		}
		break;
	case WM_NCPAINT:
		{
			return pENSB->OnNCPaint(wParam, lParam);
		}
		break;
	case WM_SIZE:
		{
			return pENSB->OnSize(wParam, lParam);
		}
		break;
	}

	return ::CallWindowProc(pENSB->m_pWndProcOld, hWnd, iMsg, wParam, lParam);
}

BOOL CEmbededNSB::ENSB_Initialize( HWND hWnd, IScrollBarUI* pNSBUI_Horz, IScrollBarUI* pNSBUI_Vert, 
								BOOL bHandleHorz /*= FALSE*/, BOOL bHandleVert /*= FALSE*/)
{
	ATLASSERT(::IsWindow(hWnd));
	if(!::IsWindow(hWnd)) return FALSE;
	
	ATLASSERT(pNSBUI_Horz);
	if(!pNSBUI_Horz) return FALSE;

	ATLASSERT(pNSBUI_Vert);
	if(!pNSBUI_Vert) return FALSE;

	CEmbededNSB* pENSB = (CEmbededNSB*)::GetProp(hWnd, ENSB_PROP);
	ATLASSERT(!pENSB);
	if(pENSB) return FALSE;
	
	pENSB = new CEmbededNSB();
	pENSB->m_hWndOwner = hWnd;
	DWORD dwHandleStyleHorz = bHandleHorz ? CNScrollBar::NSBS_HANDLE : 0 ;
	DWORD dwHandleStyleVert = bHandleVert ? CNScrollBar::NSBS_HANDLE : 0 ;
	pENSB->m_pNSBHorz = new CNScrollBar(pNSBUI_Horz, CNScrollBar::NSBS_HSCROLL | dwHandleStyleHorz, 0, 0, 0);
	pENSB->m_pNSBVert = new CNScrollBar(pNSBUI_Vert, CNScrollBar::NSBS_VSCROLL | dwHandleStyleVert, 0, 0, 0);
	pENSB->m_pWndProcOld = (WNDPROC)::GetWindowLong(hWnd, GWL_WNDPROC);
	LONG lResult = ::SetWindowLong(hWnd, GWL_WNDPROC, (LONG)_ENSBProc);
	(lResult);
	BOOL bResult = ::SetProp(hWnd, ENSB_PROP, (HANDLE)pENSB);
	(bResult);

	pENSB->m_pRootContainerHorz = new CRootContainer();
	pENSB->m_pRootContainerVert = new CRootContainer();
	pENSB->m_pRootContainerHorz->SetOwner(hWnd);
	pENSB->m_pRootContainerVert->SetOwner(hWnd);
	pENSB->m_pRootContainerHorz->Add(pENSB->m_pNSBHorz);
	pENSB->m_pRootContainerVert->Add(pENSB->m_pNSBVert);

	pENSB->m_pNSBHorz->AddScrollListener(pENSB);
	pENSB->m_pNSBVert->AddScrollListener(pENSB);

	::SendMessage(hWnd, WM_SIZE, 0, 0);

	return TRUE;
}

BOOL CEmbededNSB::ENSB_Terminate( HWND hWnd )
{
	ATLASSERT(::IsWindow(hWnd));
	if(!::IsWindow(hWnd)) return FALSE;

	CEmbededNSB* pENSB = (CEmbededNSB*)::GetProp(hWnd, ENSB_PROP);
	ATLASSERT(pENSB);
	if(!pENSB) return FALSE;

	::InvalidateRect(hWnd, NULL, TRUE);
	LONG lResult = ::SetWindowLong(hWnd, GWL_WNDPROC, (LONG)pENSB->m_pWndProcOld);
	(lResult);

	if(pENSB->m_pNSBHorz)
	{
		delete pENSB->m_pNSBHorz;
		pENSB->m_pNSBHorz = NULL;
	}

	if(pENSB->m_pNSBVert)
	{
		delete pENSB->m_pNSBVert;
		pENSB->m_pNSBVert = NULL;
	}

	if(pENSB->m_pRootContainerHorz)
	{
		delete pENSB->m_pRootContainerHorz;
		pENSB->m_pRootContainerHorz = NULL;
	}

	if(pENSB->m_pRootContainerVert)
	{
		delete pENSB->m_pRootContainerVert;
		pENSB->m_pRootContainerVert = NULL;
	}

	::RemoveProp(hWnd, ENSB_PROP);

	::SetWindowPos(hWnd, NULL, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_FRAMECHANGED);
	return TRUE;
}

BOOL CEmbededNSB::ENSB_GetScrollInfo( HWND hWnd, int fnBar, LPSCROLLINFO lpsi )
{
	ATLASSERT(::IsWindow(hWnd));
	if(!::IsWindow(hWnd)) return FALSE;

	CEmbededNSB* pENSB = (CEmbededNSB*)::GetProp(hWnd, ENSB_PROP);
	if(!pENSB) return FALSE;
	if(!pENSB->m_pNSBHorz || !pENSB->m_pNSBVert) return FALSE;

	BOOL bHorz = fnBar == SB_HORZ;
	BOOL bVerz = fnBar == SB_VERT;

	if(bHorz)
		return pENSB->m_pNSBHorz->GetScrollInfo(lpsi);
	else if(bVerz) 
		return pENSB->m_pNSBVert->GetScrollInfo(lpsi);
	else 
		return FALSE;
}

int CEmbededNSB::ENSB_GetScrollPos( HWND hWnd, int code )
{
	ATLASSERT(::IsWindow(hWnd));
	if(!::IsWindow(hWnd)) return 0;

	CEmbededNSB* pENSB = (CEmbededNSB*)::GetProp(hWnd, ENSB_PROP);
	if(!pENSB) return 0;
	if(!pENSB->m_pNSBHorz || !pENSB->m_pNSBVert) return 0;

	BOOL bHorz = code == SB_HORZ;
	BOOL bVerz = code == SB_VERT;

	if(bHorz) 
		return pENSB->m_pNSBHorz->GetScrollPos();
	else if(bVerz) 
		return pENSB->m_pNSBVert->GetScrollPos();
	else 
		return 0;
}

BOOL CEmbededNSB::ENSB_GetScrollRange( HWND hWnd, int code, LPINT lpMinPos, LPINT lpMaxPos )
{
	ATLASSERT(::IsWindow(hWnd));
	if(!::IsWindow(hWnd)) return FALSE;

	CEmbededNSB* pENSB = (CEmbededNSB*)::GetProp(hWnd, ENSB_PROP);
	if(!pENSB) return FALSE;
	if(!pENSB->m_pNSBHorz || !pENSB->m_pNSBVert) return FALSE;

	BOOL bHorz = code == SB_HORZ;
	BOOL bVerz = code == SB_VERT;

	if(bHorz) 
	{
		pENSB->m_pNSBHorz->GetScrollRange(lpMinPos, lpMaxPos);
		return TRUE;
	}
	else if(bVerz) 
	{
		pENSB->m_pNSBVert->GetScrollRange(lpMinPos, lpMaxPos);
		return TRUE;
	}
	else 
		return FALSE;
}

int CEmbededNSB::ENSB_SetScrollInfo( HWND hWnd, int fnBar, LPSCROLLINFO lpsi, BOOL fRedraw /*= TRUE*/)
{
	ATLASSERT(::IsWindow(hWnd));
	if(!::IsWindow(hWnd)) return 0;

	CEmbededNSB* pENSB = (CEmbededNSB*)::GetProp(hWnd, ENSB_PROP);
	if(!pENSB) return 0;
	if(!pENSB->m_pNSBHorz || !pENSB->m_pNSBVert) return 0;

	BOOL bHorz = fnBar == SB_HORZ;
	BOOL bVerz = fnBar == SB_VERT;

	int nResult;

	if(bHorz) 
		nResult = pENSB->m_pNSBHorz->SetScrollInfo(lpsi, fRedraw);
	else if(bVerz) 
		nResult = pENSB->m_pNSBVert->SetScrollInfo(lpsi, fRedraw);
	else 
		nResult = 0;

	::SendMessage(hWnd, WM_NCPAINT, 1, 1);
	return nResult;
}

int CEmbededNSB::ENSB_SetScrollPos( HWND hWnd, int code, int nPos, BOOL fRedraw /*= TRUE*/)
{
	ATLASSERT(::IsWindow(hWnd));
	if(!::IsWindow(hWnd)) return 0;

	CEmbededNSB* pENSB = (CEmbededNSB*)::GetProp(hWnd, ENSB_PROP);
	if(!pENSB) return 0;
	if(!pENSB->m_pNSBHorz || !pENSB->m_pNSBVert) return 0;

	BOOL bHorz = code == SB_HORZ;
	BOOL bVerz = code == SB_VERT;

	int nResult;

	if(bHorz) 
		nResult = pENSB->m_pNSBHorz->SetScrollPos(nPos, fRedraw);
	else if(bVerz) 
		nResult = pENSB->m_pNSBVert->SetScrollPos(nPos, fRedraw);
	else 
		nResult = 0;

	::SendMessage(hWnd, WM_NCPAINT, 1, 1);
	return nResult;
}

int CEmbededNSB::ENSB_SetScrollRange( HWND hWnd, int code, int nMinPos, int nMaxPos, BOOL fRedraw /*= TRUE*/)
{
	ATLASSERT(::IsWindow(hWnd));
	if(!::IsWindow(hWnd)) return 0;

	CEmbededNSB* pENSB = (CEmbededNSB*)::GetProp(hWnd, ENSB_PROP);
	if(!pENSB) return 0;
	if(!pENSB->m_pNSBHorz || !pENSB->m_pNSBVert) return 0;

	BOOL bHorz = code == SB_HORZ;
	BOOL bVerz = code == SB_VERT;

	int nResult;

	if(bHorz) 
	{
		pENSB->m_pNSBHorz->SetScrollRange(nMinPos, nMaxPos, fRedraw);
		nResult = 1;
	}
	else if(bVerz) 
	{
		pENSB->m_pNSBVert->SetScrollRange(nMinPos, nMaxPos, fRedraw);
		nResult = 1;
	}
	else 
		nResult = 0;

	::SendMessage(hWnd, WM_NCPAINT, 1, 1);
	return nResult;
}

BOOL CEmbededNSB::ENSB_Enable(HWND hWnd, int code, BOOL bEnable)
{
	ATLASSERT(::IsWindow(hWnd));
	if(!::IsWindow(hWnd)) return FALSE;

	CEmbededNSB* pENSB = (CEmbededNSB*)::GetProp(hWnd, ENSB_PROP);
	if(!pENSB) return FALSE;
	if(!pENSB->m_pNSBHorz || !pENSB->m_pNSBVert) return FALSE;

	BOOL bHorz = code == SB_HORZ;
	BOOL bVerz = code == SB_VERT;

	BOOL bResult;

	if(bHorz) 
	{
		pENSB->m_pNSBHorz->Enable(bEnable);
		bResult = TRUE;
	}
	else if(bVerz)
	{
		pENSB->m_pNSBVert->Enable(bEnable);
		bResult = TRUE;
	}
	else 
		bResult = FALSE;

	::SendMessage(hWnd, WM_NCPAINT, 1, 1);
	return bResult;
}

BOOL CEmbededNSB::ENSB_EnableHandle(HWND hWnd, int code, BOOL bEnable)
{
	ATLASSERT(::IsWindow(hWnd));
	if(!::IsWindow(hWnd)) return FALSE;

	CEmbededNSB* pENSB = (CEmbededNSB*)::GetProp(hWnd, ENSB_PROP);
	if(!pENSB) return FALSE;
	if(!pENSB->m_pNSBHorz || !pENSB->m_pNSBVert) return FALSE;

	BOOL bHorz = code == SB_HORZ;
	BOOL bVerz = code == SB_VERT;

	BOOL bResult;

	if(bHorz) 
	{
		DWORD dwHandle = 0;
		
		if(bEnable)
		{
			dwHandle = CNScrollBar::NSBS_HANDLE;
		}

		pENSB->m_pNSBHorz->ModifyStyle(CNScrollBar::NSBS_HSCROLL | dwHandle);
		bResult = TRUE;
	}
	else if(bVerz)
	{
		DWORD dwHandle = 0;

		if(bEnable)
		{
			dwHandle = CNScrollBar::NSBS_HANDLE;
		}

		pENSB->m_pNSBVert->ModifyStyle(CNScrollBar::NSBS_VSCROLL | dwHandle);
		bResult = TRUE;
	}
	else 
		bResult = FALSE;

	::SendMessage(hWnd, WM_NCPAINT, 1, 1);
	return bResult;
}

LRESULT CEmbededNSB::OnNCHitTest(WPARAM wParam, LPARAM lParam)
{
	m_lrHitTest = ::CallWindowProc(m_pWndProcOld, m_hWndOwner, WM_NCHITTEST, wParam, lParam);
	return m_lrHitTest;
}

LRESULT CEmbededNSB::OnLBtnDn(WPARAM wParam, LPARAM lParam)
{
	if(m_lrHitTest == HTHSCROLL || m_lrHitTest == HTVSCROLL)
	{
		int xPos = GET_X_LPARAM(lParam);
		int yPos = GET_Y_LPARAM(lParam);

		CPoint ptOrg(0, 0);
		::ClientToScreen(m_hWndOwner, &ptOrg);

		lParam = MAKELPARAM(xPos + ptOrg.x, yPos + ptOrg.y);

		return ::CallWindowProc(CEmbededNSB::_ENSBProc, m_hWndOwner, WM_NCLBUTTONDOWN, wParam, lParam);
	}

	return ::CallWindowProc(m_pWndProcOld, m_hWndOwner, WM_LBUTTONDOWN, wParam, lParam);
}

LRESULT CEmbededNSB::OnNCLBtnDn(WPARAM wParam, LPARAM lParam)
{
	if(m_lrHitTest == HTHSCROLL || m_lrHitTest == HTVSCROLL)
	{
		int xPos = GET_X_LPARAM(lParam);
		int yPos = GET_Y_LPARAM(lParam);
		CRect rcWnd;
		::GetWindowRect(m_hWndOwner, &rcWnd);

		LRESULT lResult;
		lParam = MAKELPARAM(xPos - rcWnd.left, yPos - rcWnd.top);

		if(m_lrHitTest == HTHSCROLL)
		{
			m_pRootContainerHorz->ProcessWindowMessage(m_hWndOwner, WM_LBUTTONDOWN, MK_LBUTTON, lParam, lResult);
		}
		
		if(m_lrHitTest == HTVSCROLL)
		{
			m_pRootContainerVert->ProcessWindowMessage(m_hWndOwner, WM_LBUTTONDOWN, MK_LBUTTON, lParam, lResult);
		}

		::SendMessage(m_hWndOwner, WM_NCPAINT, 1, 1);
		return S_OK;
	}
	
	return ::CallWindowProc(m_pWndProcOld, m_hWndOwner, WM_NCLBUTTONDOWN, wParam, lParam);
}

LRESULT	CEmbededNSB::OnLBtnUp(WPARAM wParam, LPARAM lParam)
{
	if(m_lrHitTest == HTHSCROLL || m_lrHitTest == HTVSCROLL)
	{
		int xPos = GET_X_LPARAM(lParam);
		int yPos = GET_Y_LPARAM(lParam);
		CRect rcWnd;
		::GetWindowRect(m_hWndOwner, &rcWnd);

		CPoint ptOrg(0, 0);
		::ClientToScreen(m_hWndOwner, &ptOrg);

		lParam = MAKELPARAM(xPos - rcWnd.left + ptOrg.x, yPos - rcWnd.top + ptOrg.y);
		LRESULT lResult;

		if(m_lrHitTest == HTHSCROLL)
		{
			m_pRootContainerHorz->ProcessWindowMessage(m_hWndOwner, WM_LBUTTONUP, MK_LBUTTON, lParam, lResult);
		}

		if(m_lrHitTest == HTVSCROLL)
		{
			m_pRootContainerVert->ProcessWindowMessage(m_hWndOwner, WM_LBUTTONUP, MK_LBUTTON, lParam, lResult);
		}
	}
	
	::SendMessage(m_hWndOwner, WM_NCPAINT, 1, 1);
	return ::CallWindowProc(m_pWndProcOld, m_hWndOwner, WM_LBUTTONUP, wParam, lParam);
}

LRESULT	CEmbededNSB::OnMouseMove(WPARAM wParam, LPARAM lParam)
{
	if(m_lrHitTest == HTHSCROLL || m_lrHitTest == HTVSCROLL)
	{
		int xPos = GET_X_LPARAM(lParam);
		int yPos = GET_Y_LPARAM(lParam);
		CRect rcWnd;
		::GetWindowRect(m_hWndOwner, &rcWnd);

		CPoint ptOrg(0, 0);
		::ClientToScreen(m_hWndOwner, &ptOrg);

		lParam = MAKELPARAM(xPos - rcWnd.left + ptOrg.x, yPos - rcWnd.top + ptOrg.y);
		LRESULT lResult;

		m_pRootContainerHorz->ProcessWindowMessage(m_hWndOwner, WM_MOUSEMOVE, m_lrHitTest, lParam, lResult);
		m_pRootContainerVert->ProcessWindowMessage(m_hWndOwner, WM_MOUSEMOVE, m_lrHitTest, lParam, lResult);
		
		return S_OK;
	}
	
	return ::CallWindowProc(m_pWndProcOld, m_hWndOwner, WM_MOUSEMOVE, wParam, lParam);
}

LRESULT	CEmbededNSB::OnNCMouseMove(WPARAM wParam, LPARAM lParam)
{
	int xPos = GET_X_LPARAM(lParam);
	int yPos = GET_Y_LPARAM(lParam);
	CRect rcWnd;
	::GetWindowRect(m_hWndOwner, &rcWnd);

	CPoint ptOrg(0, 0);
	::ClientToScreen(m_hWndOwner, &ptOrg);

	lParam = MAKELPARAM(xPos - rcWnd.left, yPos - rcWnd.top);
	LRESULT lResult;

	m_pRootContainerHorz->ProcessWindowMessage(m_hWndOwner, WM_MOUSEMOVE, m_lrHitTest, lParam, lResult);
	m_pRootContainerVert->ProcessWindowMessage(m_hWndOwner, WM_MOUSEMOVE, m_lrHitTest, lParam, lResult);
	return ::CallWindowProc(m_pWndProcOld, m_hWndOwner, WM_MOUSEMOVE, wParam, lParam);
}

LRESULT	CEmbededNSB::OnNCLBtnDblClk(WPARAM wParam, LPARAM lParam)
{
	if(m_lrHitTest == HTHSCROLL || m_lrHitTest == HTVSCROLL) return S_FALSE;

	return ::CallWindowProc(m_pWndProcOld, m_hWndOwner, WM_NCLBUTTONDBLCLK, wParam, lParam);
}

LRESULT	CEmbededNSB::OnNCRBtnDn(WPARAM wParam, LPARAM lParam)
{
	if(m_lrHitTest == HTHSCROLL || m_lrHitTest == HTVSCROLL) return S_FALSE;

	return ::CallWindowProc(m_pWndProcOld, m_hWndOwner, WM_NCRBUTTONDOWN, wParam, lParam);
}

LRESULT	CEmbededNSB::OnNCRBtnDblClk(WPARAM wParam, LPARAM lParam)
{
	if(m_lrHitTest == HTHSCROLL || m_lrHitTest == HTVSCROLL) return S_FALSE;

	return ::CallWindowProc(m_pWndProcOld, m_hWndOwner, WM_NCRBUTTONDBLCLK, wParam, lParam);
}

LRESULT	CEmbededNSB::OnNCPaint(WPARAM wParam, LPARAM lParam)
{	
	LRESULT lResult;

	if(lParam == 1)
	{
		CRect rcWnd;
		::GetWindowRect(m_hWndOwner, &rcWnd);
		HRGN hRgnWnd = ::CreateRectRgn(0, 0, rcWnd.Width(), rcWnd.Height());
		HRGN hRgnHSB = ::CreateRectRgn(m_rcNSBHorz.left, m_rcNSBHorz.top, m_rcNSBHorz.right, m_rcNSBHorz.bottom);
		HRGN hRgnVSB = ::CreateRectRgn(m_rcNSBVert.left, m_rcNSBVert.top, m_rcNSBVert.right, m_rcNSBVert.bottom);
		HRGN hRgnClip = ::CreateRectRgn(0, 0, 0, 0);
		HRGN hRgnClip2 = ::CreateRectRgn(0, 0, 0, 0);

		::CombineRgn(hRgnClip, hRgnWnd, hRgnHSB, RGN_DIFF);
		::CombineRgn(hRgnClip2, hRgnClip, hRgnVSB, RGN_DIFF);
		wParam = (WPARAM)hRgnClip2;

		::DeleteObject(hRgnWnd);
		::DeleteObject(hRgnHSB);
		::DeleteObject(hRgnVSB);
		::DeleteObject(hRgnClip);
		::DeleteObject(hRgnClip2);
	}

	lResult = ::CallWindowProc(m_pWndProcOld, m_hWndOwner, WM_NCPAINT, wParam, lParam);

	HDC hWndDC = ::GetWindowDC(m_hWndOwner);

	POINT ptOrg = {0, 0};

	CRect rcBoundRootHorz;
	m_pRootContainerHorz->GetBounds(&rcBoundRootHorz);
	OffsetViewportOrgEx(hWndDC, rcBoundRootHorz.left, rcBoundRootHorz.top, &ptOrg);
	m_pRootContainerHorz->OnDraw(hWndDC);
	SetViewportOrgEx(hWndDC, ptOrg.x, ptOrg.y, NULL);

	CRect rcBoundRootVert;
	m_pRootContainerVert->GetBounds(&rcBoundRootVert);
	OffsetViewportOrgEx(hWndDC, rcBoundRootVert.left, rcBoundRootVert.top, &ptOrg);
	m_pRootContainerVert->OnDraw(hWndDC);
	SetViewportOrgEx(hWndDC, ptOrg.x, ptOrg.y, NULL);

	::ReleaseDC(m_hWndOwner, hWndDC);

	return lResult;
}

LRESULT	CEmbededNSB::OnSize(WPARAM wParam, LPARAM lParam)
{
	LRESULT lResult = ::CallWindowProc(m_pWndProcOld, m_hWndOwner, WM_SIZE, wParam, lParam);

	LONG lStyle = ::GetWindowLong(m_hWndOwner, GWL_STYLE);
	BOOL bHorz = lStyle & WS_HSCROLL;
	BOOL bVert = lStyle & WS_VSCROLL;

	CRect rcWnd;
	::GetWindowRect(m_hWndOwner, &rcWnd);
	CPoint ptClientOrg(0, 0);
	::ClientToScreen(m_hWndOwner, &ptClientOrg);
	ptClientOrg.Offset(-rcWnd.left, -rcWnd.top);
	rcWnd.OffsetRect(-rcWnd.left, -rcWnd.top);

	CRect rcNSBHorz, rcNSBVert;
	::CopyRect(&rcNSBHorz, &rcWnd);
	::CopyRect(&rcNSBVert, &rcWnd);

	int nSBWidth = ::GetSystemMetrics(SM_CYHSCROLL);
	int nBorder = ::GetSystemMetrics(SM_CXSIZEFRAME);

	if(bHorz && bVert)
	{
		rcNSBHorz.top = rcNSBHorz.bottom - nSBWidth;
		rcNSBHorz.right -= nSBWidth;
		rcNSBHorz.InflateRect(-nBorder, 0);
		rcNSBHorz.OffsetRect(0, -nBorder);

		rcNSBVert.left = rcNSBVert.right - nSBWidth;
		rcNSBVert.top += ptClientOrg.y - nBorder;
		rcNSBVert.bottom -= nSBWidth;
		rcNSBVert.InflateRect(0, -nBorder);
		rcNSBVert.OffsetRect(-nBorder, 0);
	}
	else if(!bHorz && bVert)
	{
		rcNSBHorz.SetRectEmpty();

		rcNSBVert.left = rcNSBVert.right - nSBWidth;
		rcNSBVert.top += ptClientOrg.y - nBorder;
		rcNSBVert.InflateRect(0, -nBorder);
		rcNSBVert.OffsetRect(-nBorder, 0 );
	}
	else if(bHorz && !bVert)
	{
		rcNSBHorz.top = rcNSBHorz.bottom - nSBWidth;	
		rcNSBHorz.InflateRect(-nBorder, 0);
		rcNSBHorz.OffsetRect(0, -nBorder);

		rcNSBVert.SetRectEmpty();
	}
	else if(!bHorz && !bVert)
	{
		rcNSBHorz.SetRectEmpty();
		rcNSBVert.SetRectEmpty();
	}

	m_rcNSBHorz = rcNSBHorz;
	m_rcNSBVert = rcNSBVert;
	m_pRootContainerHorz->MoveWindow(&m_rcNSBHorz);
	m_pRootContainerVert->MoveWindow(&m_rcNSBVert);

	m_pNSBHorz->MoveWindow(0, 0, rcNSBHorz.Width(), rcNSBHorz.Height());
	m_pNSBVert->MoveWindow(0, 0, rcNSBVert.Width(), rcNSBVert.Height());

	SendMessage(m_hWndOwner, WM_NCPAINT, 1, 1);

	return lResult;
}

void CEmbededNSB::OnHScroll( CScrollEvent* pScrollEvent /*= NULL*/ )
{
	ATLASSERT(m_pNSBHorz);
	if(!m_pNSBHorz) return;

	WPARAM wParam = MAKEWPARAM(pScrollEvent->m_nSBCode, pScrollEvent->m_nPos);
	LPARAM lParam = (LPARAM)m_pNSBHorz;
	::SendMessage(m_hWndOwner, WM_HSCROLL, wParam, lParam);
}

void CEmbededNSB::OnVScroll( CScrollEvent* pScrollEvent /*= NULL*/ )
{
	ATLASSERT(m_pNSBVert);
	if(!m_pNSBVert) return;

	WPARAM wParam = MAKEWPARAM(pScrollEvent->m_nSBCode, pScrollEvent->m_nPos);
	LPARAM lParam = (LPARAM)m_pNSBVert;
	::SendMessage(m_hWndOwner, WM_VSCROLL, wParam, lParam);
}